use std::sync::mpsc::channel;
use ehttp::Response;
use ehttp::streaming::Part;


pub struct HttpState {
    pub channel: Option<std::sync::mpsc::Receiver<HttpChannelChunk>>,
    pub request_in_progress: bool,
    pub total_size: f32,
    pub current_size: f32,
    pub data: Vec<u8>,
}

pub enum HttpChannelChunk {
    Start(f32),
    Data(Vec<u8>),
    End,
    Oneshot(Response)
}


impl HttpState {
    pub fn new() -> HttpState {
        HttpState {
            channel: None,
            request_in_progress: false,
            total_size: 0.0,
            current_size: 0.0,
            data: vec![],
        }
    }

    pub fn do_request(&mut self, req: ehttp::Request) {
        if !self.request_in_progress {
            self.request_in_progress = true;

            let (sender, receiver) = channel();
            self.channel = Some(receiver);

            ehttp::streaming::fetch(req, move |res| {
                let part = match res {
                    Ok(part) => part,
                    Err(err) => {
                        eprintln!("waaa waaa waaa {err}");
                        return std::ops::ControlFlow::Break(());
                    }
                };

                match part {
                    Part::Response(response) => {
                        if response.ok {
                            let total_size = response.headers.get("Content-Length").expect("Missing Content-Length").parse().expect("Content-Length wasn't a number");

                            sender.send(HttpChannelChunk::Start(total_size)).expect("Unable to send start chunk");

                            std::ops::ControlFlow::Continue(())
                        } else {
                            std::ops::ControlFlow::Break(())
                        }
                    }
                    Part::Chunk(chunk) => {
                        if chunk.is_empty() {
                            sender.send(HttpChannelChunk::End).expect("Unable to send end chunk");
                        } else {
                            sender.send(HttpChannelChunk::Data(chunk)).expect("Unable to send data chunk");
                        }

                        std::ops::ControlFlow::Continue(())
                    }
                }
            });
        }
    }

    pub fn do_oneshot_request(&mut self, req: ehttp::Request)  {
        self.request_in_progress = true;

        let (sender, receiver) = channel();
        self.channel = Some(receiver);

        ehttp::fetch(req, move |res| {
            sender.send(HttpChannelChunk::Oneshot(res.expect("Invalid response"))).expect("Failed to send oneshot response")
        })
    }
}