use keyring::Entry;
use serde_derive::{Deserialize, Serialize};

#[derive(Serialize, Deserialize, Clone)]
pub struct Account {
    pub session_id: String,
    pub character_id: String,
    pub character_name: String,
}

impl Account {
    pub fn get_secrets() -> Option<Self> {
        let session_id = Entry::new("osrs-luncher", "session-id").expect("Unable to get session_id entry");

        if session_id.get_password().is_err() {
            return None;
        }

        let character_id = Entry::new("osrs-luncher", "character-id").expect("Unable to get character_id entry");
        let character_name = Entry::new("osrs-luncher", "character-name").expect("Unable to get character_name entry");

        Some(Account {
            session_id: session_id.get_password().expect("Unable to get session_id entry content"),
            character_id: character_id.get_password().expect("Unable to get character_id entry content"),
            character_name: character_name.get_password().expect("Unable to get character_name entry content"),
        })
    }

    pub fn insert_secrets(&self) -> bool {
        let session_id = Entry::new("osrs-luncher", "session-id").expect("Unable to get session_id entry");
        let character_id = Entry::new("osrs-luncher", "character-id").expect("Unable to get character_id entry");
        let character_name = Entry::new("osrs-luncher", "character-name").expect("Unable to get character_name entry");

        session_id.set_password(self.session_id.as_str()).expect("Unable to set session_id");
        character_id.set_password(self.character_id.as_str()).expect("Unable to set character_id");
        character_name.set_password(self.character_name.as_str()).expect("Unable to set character_name");

        true
    }

    pub fn delete_secrets(&self) -> bool {
        let session_id = Entry::new("osrs-luncher", "session-id").expect("Unable to get session_id entry").delete_credential().is_ok();
        let character_id = Entry::new("osrs-luncher", "character-id").expect("Unable to get character_id entry").delete_credential().is_ok();
        let character_name = Entry::new("osrs-luncher", "character-name").expect("Unable to get character_name entry").delete_credential().is_ok();

        session_id | character_id | character_name
    }
}