use egui::{Context, Ui};
use egui_phosphor::fill;
use rfd::FileDialog;
use crate::backend::config::Config;
use crate::ui::launcher::Launcher;
use crate::ui::Pages;

#[derive(Clone)]
pub struct Settings {
}

impl Settings {
    pub fn init() -> Settings {
        Settings {}
    }

    pub fn update(&mut self,
                  next_page: &mut Option<Pages>,
                  config: &mut Config,
                  _ctx: &Context,
                  ui: &mut Ui
    ) {
        ui.heading("Settings");

        ui.button(format!("{} Open Data Folder", fill::FOLDER)).clicked();
        
        ui.horizontal(|ui| {
            ui.label("RuneLite path: ");
            ui.add(egui::TextEdit::singleline(&mut config.paths.runelite_path));

            if ui.button(format!("{} Select", fill::FOLDER)).clicked() {
                // TODO: switch this to use a channel and another thread
                config.paths.runelite_path = FileDialog::new().pick_file().unwrap_or_default().into_os_string().into_string().unwrap_or_default();
            }
        });

        ui.horizontal(|ui| {
            ui.label("Java path: ");
            ui.add(egui::TextEdit::singleline(&mut config.paths.java_path));

            if ui.button(format!("{} Select", fill::FOLDER)).clicked() {
                // TODO: switch this to use a channel and another thread
                config.paths.java_path = FileDialog::new().pick_file().unwrap_or_default().into_os_string().into_string().unwrap_or_default();
            }
        });

        if ui.button("Save and Go Back").clicked() {
            config.save_config();

            *next_page = Some(Pages::Launcher(Launcher::init()))
        }
    }
}